@@ -182,11 +182,25 @@ typedef struct GProc {
     int pid;
 } GProc;
 
-typedef enum displayStatus { notRunning = 0, running, zombie, phoenix, raiser,
-                             textMode, reserve, remoteLogin } DisplayStatus;
+typedef enum displayStatus {
+    notRunning = 0, /* waiting for being started */
+    running,        /* was started */
+    zombie,         /* manager and server killed, remove/suspend when both are gone */
+    phoenix,        /* server killed, restart when it dies */
+    raiser,         /* manager killed, restart when it dies */
+    textMode,       /* suspended, console mode */
+    reserve,        /* suspended, reserve display */
+    remoteLogin     /* running X -query */
+} DisplayStatus;
 
-typedef enum serverStatus { ignore = 0, awaiting, starting,
-                            terminated, killed, pausing } ServerStatus;
+typedef enum serverStatus {
+    ignore = 0,     /* error in this state is no error */
+    awaiting,       /* waking for being started */
+    starting,       /* process launched, wait max serverTimeout secs */
+    terminated,     /* process SIGTERMed, wait max serverTimeout secs */
+    killed,         /* process SIGKILLed, wait max 10 secs */
+    pausing         /* startup failed, wait openDelay secs */
+} ServerStatus;
 
 typedef struct {
     unsigned how:2,    /* 0=none 1=reboot 2=halt (SHUT_*) */
@@ -285,8 +299,6 @@ struct display {
 
     CONF_CORE_LOCAL_DEFS
 
-    int idleTimeout;            /* abort login after that time */
-
     /* information potentially derived from resources */
     Xauth **authorizations;     /* authorization data */
     int authNum;                /* number of authorizations */
@@ -418,7 +430,7 @@ struct display
 int anyRunningDisplays(void);
 int anyReserveDisplays(void);
 int idleReserveDisplays(void);
-int startReserveDisplay(int lt);
+int startReserveDisplay(void);
 const char *displayName(struct display *);
 
 /* in reset.c */
@@ -471,6 +483,8 @@ FILE *pOpen(char **what, char m, volatile int *pid);
 int pClose(FILE *f, volatile int *pid);
 char *locate(const char *exe);
 void terminateProcess(int pid, int sig);
+void blockTerm(void);
+void unblockTerm(void);
 
 void gSet(GTalk *talk); /* call before gOpen! */
 void gCloseOnExec(GPipe *pajp);
