@@ -60,7 +60,6 @@
 
 #include "treeview.h"
 #include "treeview.moc"
-#include "khotkeys.h"
 #include "menufile.h"
 #include "menuinfo.h"
 
@@ -161,7 +160,6 @@ TreeView::TreeView( KActionCollection *ac, QWidget *parent, const char *name )
       m_layoutDirty(false)
 {
     setObjectName(name);
-	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
     setAllColumnsShowFocus(true);
     setRootIsDecorated(true);
     setSorting(-1);
@@ -319,9 +317,9 @@ QString TreeView::findName(KDesktopFile *df, bool deleted)
           QString res = df->resource();
 
           bool isLocal = true;
-          QStringList files = KGlobal::dirs()->findAllResources(res.toLatin1(), file);
-          for(QStringList::ConstIterator it = files.begin();
-              it != files.end();
+          const QStringList files = KGlobal::dirs()->findAllResources(res.toLatin1(), file);
+          for(QStringList::ConstIterator it = files.constBegin();
+              it != files.constEnd();
               ++it)
           {
              if (isLocal)
@@ -594,8 +592,8 @@ QStringList TreeView::fileList(const QString& rPath)
     QStringList filelist;
 
     // loop through all resource dirs and build a file list
-    QStringList resdirlist = KGlobal::dirs()->resourceDirs("apps");
-    for (QStringList::ConstIterator it = resdirlist.begin(); it != resdirlist.end(); ++it)
+    const QStringList resdirlist = KGlobal::dirs()->resourceDirs("apps");
+    for (QStringList::ConstIterator it = resdirlist.constBegin(); it != resdirlist.constEnd(); ++it)
     {
         QDir dir((*it) + '/' + relativePath);
         if(!dir.exists()) continue;
@@ -604,8 +602,8 @@ QStringList TreeView::fileList(const QString& rPath)
         dir.setNameFilters(QStringList() << "*.desktop;*.kdelnk");
 
         // build a list of files
-        QStringList files = dir.entryList();
-        for (QStringList::ConstIterator it = files.begin(); it != files.end(); ++it) {
+        const QStringList files = dir.entryList();
+        for (QStringList::ConstIterator it = files.constBegin(); it != files.constEnd(); ++it) {
             // does not work?!
             //if (filelist.contains(*it)) continue;
 
@@ -633,16 +631,16 @@ QStringList TreeView::dirList(const QString& rPath)
     QStringList dirlist;
 
     // loop through all resource dirs and build a subdir list
-    QStringList resdirlist = KGlobal::dirs()->resourceDirs("apps");
-    for (QStringList::ConstIterator it = resdirlist.begin(); it != resdirlist.end(); ++it)
+    const QStringList resdirlist = KGlobal::dirs()->resourceDirs("apps");
+    for (QStringList::ConstIterator it = resdirlist.constBegin(); it != resdirlist.constEnd(); ++it)
     {
         QDir dir((*it) + '/' + relativePath);
         if(!dir.exists()) continue;
         dir.setFilter(QDir::Dirs);
 
         // build a list of subdirs
-        QStringList subdirs = dir.entryList();
-        for (QStringList::ConstIterator it = subdirs.begin(); it != subdirs.end(); ++it) {
+        const QStringList subdirs = dir.entryList();
+        for (QStringList::ConstIterator it = subdirs.constBegin(); it != subdirs.constEnd(); ++it) {
             if ((*it) == "." || (*it) == "..") continue;
             // does not work?!
             // if (dirlist.contains(*it)) continue;
