@@ -35,12 +35,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/Xutil.h>
 #include <fixx11h.h>
 
+#include <QScriptValue>
+#include "scripting/client.h"
+
 #include "utils.h"
 #include "options.h"
 #include "workspace.h"
 #include "kdecoration.h"
 #include "rules.h"
 #include "toplevel.h"
+#include "clientgroup.h"
 
 #ifdef HAVE_XSYNC
 #include <X11/extensions/sync.h>
@@ -52,8 +56,19 @@ class QProcess;
 class QTimer;
 class KStartupInfoData;
 
+namespace SWrapper {
+class Client;
+}
+
+typedef QPair<SWrapper::Client*, QScriptValue> ClientResolution;
+
 namespace KWin
 {
+namespace TabBox {
+
+class TabBoxClientImpl;
+}
+
 
 class Workspace;
 class Client;
@@ -99,6 +114,13 @@ class Client
          */
         bool isSpecialWindow() const;
         bool hasNETSupport() const;
+	
+        /**
+          * This is a public object with no wrappers or anything to keep it fast,
+          * so in essence, direct access is allowed. Please be very careful while
+          * using this object
+          */
+	QHash<QScriptEngine*, ClientResolution>* scriptCache;
 
         QSize minSize() const;
         QSize maxSize() const;
@@ -128,7 +150,10 @@ class Client
         QSize adjustedSize() const;
 
         QPixmap icon() const;
+        QPixmap icon( const QSize& size ) const;
         QPixmap miniIcon() const;
+        QPixmap bigIcon() const;
+        QPixmap hugeIcon() const;
 
         bool isActive() const;
         void setActive( bool );
@@ -137,6 +162,12 @@ class Client
         void setDesktop( int );
         void setOnAllDesktops( bool set );
 
+        virtual QStringList activities() const;
+        void setOnActivity( const QString &activity, bool enable );
+        void setOnAllActivities( bool set );
+        void setOnActivities( QStringList newActivitiesList );
+        void updateActivities( bool includeTransients );
+
         /// Is not minimized and not hidden. I.e. normally visible on some virtual desktop.
         bool isShown( bool shaded_is_shown ) const;
         bool isHiddenInternal() const; // For compositing
@@ -166,6 +197,7 @@ class Client
         bool noBorder() const;
         void setNoBorder( bool set );
         bool userCanSetNoBorder() const;
+        void checkNoBorder();
 
         bool skipTaskbar( bool from_outside = false ) const;
         void setSkipTaskbar( bool set, bool from_outside );
@@ -173,6 +205,9 @@ class Client
         bool skipPager() const;
         void setSkipPager( bool );
 
+        bool skipSwitcher() const;
+        void setSkipSwitcher( bool set );
+
         bool keepAbove() const;
         void setKeepAbove( bool );
         bool keepBelow() const;
@@ -207,17 +242,30 @@ class Client
 
         void updateShape();
 
-        void setGeometry( int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet );
-        void setGeometry( const QRect& r, ForceGeometry_t force = NormalGeometrySet );
+        void setGeometry( int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet, bool emitJs = true );
+        void setGeometry( const QRect& r, ForceGeometry_t force = NormalGeometrySet, bool emitJs = true );
         void move( int x, int y, ForceGeometry_t force = NormalGeometrySet );
         void move( const QPoint& p, ForceGeometry_t force = NormalGeometrySet );
         /// plainResize() simply resizes
-        void plainResize( int w, int h, ForceGeometry_t force = NormalGeometrySet );
-        void plainResize( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
+        void plainResize( int w, int h, ForceGeometry_t force = NormalGeometrySet, bool emitJs = true );
+        void plainResize( const QSize& s, ForceGeometry_t force = NormalGeometrySet, bool emitJs = true );
         /// resizeWithChecks() resizes according to gravity, and checks workarea position
         void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
         void keepInArea( QRect area, bool partial = false );
+        void setElectricBorderMode( ElectricMaximizingMode mode );
+        ElectricMaximizingMode electricBorderMode() const;
+        void setElectricBorderMaximizing( bool maximizing );
+        bool isElectricBorderMaximizing() const;
+        QRect electricBorderMaximizeGeometry();
+
+        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
+
+        /** Set the quick tile mode ("snap") of this window.
+         * This will also handle preserving and restoring of window geometry as necessary.
+         * @param mode The tile mode (left/right) to give this window.
+         */
+        void setQuickTileMode( QuickTileMode mode, bool keyboard = false );
 
         void growHorizontal();
         void shrinkHorizontal();
@@ -228,6 +276,7 @@ class Client
         KShortcut shortcut() const;
         void setShortcut( const QString& cut );
 
+        WindowOperation mouseButtonToWindowOperation( Qt::MouseButtons button );
         bool performMouseCommand( Options::MouseCommand, const QPoint& globalPos, bool handled = false );
 
         QRect adjustedClientArea( const QRect& desktop, const QRect& area ) const;
@@ -270,7 +319,7 @@ class Client
 
         static bool belongToSameApplication( const Client* c1, const Client* c2, bool active_hack = false );
         static bool sameAppWindowRoleMatch( const Client* c1, const Client* c2, bool active_hack );
-        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon );
+        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon );
 
         void minimize( bool avoid_animation = false );
         void unminimize( bool avoid_animation = false );
@@ -285,6 +334,24 @@ class Client
         StrutRect strutRect( StrutArea area ) const;
         StrutRects strutRects() const;
         bool hasStrut() const;
+
+        // Tabbing functions
+        ClientGroup* clientGroup() const; // Returns a pointer to client_group
+        void setClientGroup( ClientGroup* group );
+        /*
+        *   If shown is true the client is mapped and raised, if false
+        *   the client is unmapped and hidden, this function is called
+        *   when the tabbing group of the client switches its visible
+        *   client.
+        */
+        void setClientShown( bool shown );
+        /*
+        *   When a click is done in the decoration and it calls the group
+        *   to change the visible client it starts to move-resize the new
+        *   client, this function stops it.
+        */
+        void dontMoveResize();
+
         /**
          * Whether or not the window has a strut that expands through the invisible area of
          * an xinerama setup where the monitors are not the same resolution.
@@ -320,6 +387,8 @@ class Client
                         QRect(0, 0, width(), height());
         }
 
+        QRect transparentRect() const;
+
         QRegion decorationPendingRegion() const;
 
         enum CoordinateMode {
@@ -329,6 +398,11 @@ class Client
         void layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom, CoordinateMode mode) const;
         virtual void addRepaintFull();
 
+        TabBox::TabBoxClientImpl* tabBoxClient() const { return m_tabBoxClient; }
+
+        //sets whether the client should be treated as a SessionInteract window
+        void setSessionInteract(bool needed);
+
     private slots:
         void autoRaise();
         void shadeHover();
@@ -370,11 +444,13 @@ class Client
         bool buttonPressEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool buttonReleaseEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool motionNotifyEvent( Window w, int state, int x, int y, int x_root, int y_root );
+        void checkQuickTilingMaximizationZones( int xroot, int yroot );
 
-        void processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root );
+        bool processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root,
+            bool ignoreMenu = false );
 
     protected:
-        virtual void debug( kdbgstream& stream ) const;
+        virtual void debug( QDebug& stream ) const;
         virtual bool shouldUnredirect() const;
 
     private slots:
@@ -384,12 +460,29 @@ class Client
         void syncTimeout();
         void delayedSetShortcut();
         void repaintDecorationPending();
-
+    
+    //Signals for the scripting interface
+    //Signals make an excellent way for communcation
+    //in between objects as compared to simple function
+    //calls
+    signals:
+	void s_clientMoved();
+	void clientManaging(KWin::Client*);
+	void s_minimized();
+	void s_unminimized();
+	void maximizeSet(QPair<bool, bool>);
+	void s_activated();
+	void s_fullScreenSet(bool, bool);
+    
+    // To make workspace-client calls, a few slots are also
+    // required
+    public slots:
+	void sl_activated();
+	
     private:
         void exportMappingState( int s ); // ICCCM 4.1.3.1, 4.1.4, NETWM 2.5.1
         bool isManaged() const; ///< Returns false if this client is not yet managed
         void updateAllowedActions( bool force = false );
-        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
         QRect fullscreenMonitorsArea( NETFullscreenMonitors topology ) const;
         void changeMaximize( bool horizontal, bool vertical, bool adjust );
         void checkMaximizeGeometry();
@@ -460,6 +553,7 @@ class Client
         KDecoration* decoration;
         Bridge* bridge;
         int desk;
+        QStringList activityList;
         bool buttonDown;
         bool moveResizeMode;
         bool move_faked_activity;
@@ -482,6 +576,12 @@ class Client
             Kept ///< The frame should be unmapped, but is kept (For compositing)
             };
         MappingState mapping_state;
+
+        /** The quick tile mode of this window.
+         */
+        QuickTileMode quick_tile_mode;
+        QRect geom_pretile;
+
         void readTransient();
         Window verifyTransientFor( Window transient_for, bool set );
         void addTransient( Client* cl );
@@ -507,6 +607,7 @@ class Client
         uint Pping : 1; ///< Does it support _NET_WM_PING?
         uint input : 1; ///< Does the window want input in its wm_hints
         uint skip_pager : 1;
+        uint skip_switcher : 1;
         uint motif_may_resize : 1;
         uint motif_may_move : 1;
         uint motif_may_close : 1;
@@ -526,6 +627,8 @@ class Client
         void getWindowProtocols();
         QPixmap icon_pix;
         QPixmap miniicon_pix;
+        QPixmap bigicon_pix;
+        QPixmap hugeicon_pix;
         QCursor cursor;
         // DON'T reorder - Saved to config files !!!
         enum FullScreenMode
@@ -546,6 +649,7 @@ class Client
         QString cap_normal, cap_iconic, cap_suffix;
         Group* in_group;
         Window window_group;
+        ClientGroup* client_group;
         Layer in_layer;
         QTimer* ping_timer;
         QProcess* process_killer;
@@ -584,8 +688,18 @@ class Client
         QTimer* demandAttentionKNotifyTimer;
         QPixmap decorationPixmapLeft, decorationPixmapRight, decorationPixmapTop, decorationPixmapBottom;
         PaintRedirector* paintRedirector;
+        TabBox::TabBoxClientImpl* m_tabBoxClient;
+
+        bool electricMaximizing;
+        ElectricMaximizingMode electricMode;
 
         friend bool performTransiencyCheck();
+        friend class SWrapper::Client;
+
+        void checkActivities();
+        bool activitiesDefined; //whether the x property was actually set
+
+        bool needsSessionInteract;
     };
 
 /**
@@ -675,6 +789,16 @@ inline Group* Client::group()
     return in_group;
     }
 
+inline void Client::setClientGroup( ClientGroup* group )
+    {
+    client_group = group;
+    }
+
+inline ClientGroup* Client::clientGroup() const
+    {
+    return client_group;
+    }
+
 inline bool Client::isMinimized() const
     {
     return minimized;
@@ -687,7 +811,8 @@ inline bool Client::isActive() const
 
 inline bool Client::isShown( bool shaded_is_shown ) const
     {
-    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden;
+    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden &&
+        ( clientGroup() == NULL || clientGroup()->visible() == this );
     }
 
 inline bool Client::isHiddenInternal() const
@@ -715,6 +840,16 @@ inline QPixmap Client::miniIcon() const
     return miniicon_pix;
     }
 
+inline QPixmap Client::bigIcon() const
+    {
+    return bigicon_pix;
+    }
+
+inline QPixmap Client::hugeIcon() const
+    {
+    return hugeicon_pix;
+    }
+
 inline QRect Client::geometryRestore() const
     {
     return geom_restore;
@@ -740,6 +875,11 @@ inline bool Client::skipPager() const
     return skip_pager;
     }
 
+inline bool Client::skipSwitcher() const
+    {
+    return skip_switcher;
+    }
+
 inline bool Client::keepAbove() const
     {
     return keep_above;
@@ -800,9 +940,9 @@ inline QRect Client::visibleRect() const
    return geometry().adjusted( -padding_left, -padding_top, padding_right, padding_bottom );
    }
 
-inline void Client::setGeometry( const QRect& r, ForceGeometry_t force )
+inline void Client::setGeometry( const QRect& r, ForceGeometry_t force, bool emitJs )
     {
-    setGeometry( r.x(), r.y(), r.width(), r.height(), force );
+    setGeometry( r.x(), r.y(), r.width(), r.height(), force, emitJs );
     }
 
 inline void Client::move( const QPoint& p, ForceGeometry_t force )
@@ -810,9 +950,9 @@ inline void Client::move( const QPoint& p, ForceGeometry_t force )
     move( p.x(), p.y(), force );
     }
 
-inline void Client::plainResize( const QSize& s, ForceGeometry_t force )
+inline void Client::plainResize( const QSize& s, ForceGeometry_t force, bool emitJs )
     {
-    plainResize( s.width(), s.height(), force );
+    plainResize( s.width(), s.height(), force, emitJs );
     }
 
 inline void Client::resizeWithChecks( const QSize& s, ForceGeometry_t force )
