@@ -105,7 +105,13 @@ public:
         NoBorderOp,
         NoOp,
         SetupWindowShortcutOp,
-        ApplicationRulesOp
+        ApplicationRulesOp,
+        RemoveClientFromGroupOp, // Remove from group
+        CloseClientGroupOp, // Close the group
+        MoveClientInGroupLeftOp, // Move left in the group
+        MoveClientInGroupRightOp, // Move right in the group
+        ToggleClientTiledStateOp, // put a floating client into tiling
+        ClientGroupDragOp
         };
     /**
      * Basic color types that should be recognized by all decoration styles.
@@ -129,12 +135,13 @@ public:
      */
     enum
         {
-        SettingDecoration = 1 << 0, ///< The decoration was changed
-        SettingColors     = 1 << 1, ///< The color palette was changed
-        SettingFont       = 1 << 2, ///< The titlebar font was changed
-        SettingButtons    = 1 << 3, ///< The button layout was changed
-        SettingTooltips   = 1 << 4, ///< The tooltip setting was changed
-        SettingBorder     = 1 << 5  ///< The border size setting was changed
+        SettingDecoration  = 1 << 0, ///< The decoration was changed
+        SettingColors      = 1 << 1, ///< The color palette was changed
+        SettingFont        = 1 << 2, ///< The titlebar font was changed
+        SettingButtons     = 1 << 3, ///< The button layout was changed
+        SettingTooltips    = 1 << 4, ///< The tooltip setting was changed
+        SettingBorder      = 1 << 5, ///< The border size setting was changed
+        SettingCompositing = 1 << 6  ///< Compositing settings was changed
         };
 
     /**
@@ -191,23 +198,46 @@ public:
                                         ///  The mask is still used to define the input region and the blurred
                                         ///  region, when the blur plugin is enabled.
                                         ///  @since 4.3
+        AbilityExtendIntoClientArea = 3002, ///< The decoration respects transparentRect()
+                                            ///  @since 4.4
+        AbilityUsesBlurBehind = 3003, ///< The decoration wants the background to be blurred, when the blur plugin is enabled.
+                                     /// @since 4.6
+        // Tabbing
+        AbilityClientGrouping = 4000, ///< The decoration supports tabbing
         // TODO colors for individual button types
         ABILITY_DUMMY = 10000000
         };
 
     enum Requirement { REQUIREMENT_DUMMY = 1000000 };
+    
+    /**
+    *
+    * Returns mimeType used to drag and drop clientGroupItems 
+    */
+    
+    static QString clientGroupItemDragMimeType();
+
 };
 
-/**
- * Decoration shadow type.
- */
-enum ShadowType
+class KWIN_EXPORT ClientGroupItem
     {
-    ShadowBorderedActive = 0, ///< Active shadow of decorated windows
-    ShadowBorderedInactive, ///< Inctive shadow of decorated windows
-    ShadowBorderlessActive, ///< Active shadow of undecorated windows
-    ShadowBorderlessInactive, ///< Inctive shadow of undecorated windows
-    ShadowOther ///< Shadow of all other windows (Menus, comboboxes, tooltips, etc.)
+    public:
+        ClientGroupItem( QString t, QIcon i )
+            {
+            title_ = t;
+            icon_ = i;
+            }
+        inline QIcon icon() const
+            {
+            return icon_;
+            }
+        inline QString title() const
+            {
+            return title_;
+            }
+    private:
+        QString title_;
+        QIcon icon_;
     };
 
 class KDecorationProvides
@@ -299,7 +329,7 @@ public:
      * in titleButtonsLeft().
      *
      * The default ( which is also returned if customButtonPositions returns false )
-     * is "HIAX".
+     * is "HIA__X".
      * Unknown buttons in the returned string must be ignored.
      * The changed flags for this setting is SettingButtons.
      */
@@ -603,6 +633,20 @@ class KWIN_EXPORT KDecoration
          * Convenience function that returns the height of the decoration.
          */
         int height() const;  // convenience
+        /**
+         * Returns the rectangle within the window frame that should be transparent.
+         * Usually this rectangle is the same as the client area, and it is never
+         * larger.
+         *
+         * If the client has requested that the window frame is extended into the
+         * client area, this rectangle is smaller than the client area.
+         *
+         * If the window frame should cover the whole client area, a null rectangle
+         * is returned.
+         *
+         * @since 4.4
+         */
+        QRect transparentRect() const;
         /**
          * This function is the default handler for mouse events. All mouse events
          * that are not handled by the decoration itself should be passed to it
@@ -683,6 +727,7 @@ class KWIN_EXPORT KDecoration
          * isShade() to get the current state.
          */
         virtual void shadeChange() = 0;
+
     Q_SIGNALS:
         /**
          * This signal is emitted whenever the window's keep-above state changes.
@@ -692,6 +737,7 @@ class KWIN_EXPORT KDecoration
          * This signal is emitted whenever the window's keep-below state changes.
          */
         void keepBelowChanged( bool );
+
     public:
         /**
          * This function may be reimplemented to provide custom bound drawing
@@ -859,6 +905,7 @@ class KWIN_EXPORT KDecoration
          * @internal
          */
         void emitKeepBelowChanged( bool below );
+
     private:
         KDecorationBridge* bridge_;
         QWidget* w_;
@@ -867,6 +914,7 @@ class KWIN_EXPORT KDecoration
         friend class KDecorationUnstable; // for bridge_
         static KDecorationOptions* options_;
         KDecorationPrivate* d;
+
     };
 
 /**
@@ -895,6 +943,67 @@ class KWIN_EXPORT KDecorationUnstable
          * Returns @a true if compositing--and therefore ARGB--is enabled.
          */
         bool compositingActive() const;
+
+        // Window tabbing
+
+        /**
+         * Returns whether or not this client group contains the active client.
+         */
+        bool isClientGroupActive();
+        /**
+         * Return a list of all the clients in the group that contains the client that this
+         * decoration is attached to.
+         */
+        QList< ClientGroupItem > clientGroupItems() const;
+        /**
+         * Returns a unique identifier for the client at index \p index of the client group list.
+         * \see moveItemToClientGroup()
+         */
+        long itemId( int index );
+        /**
+         * Returns the list index of the currently visible client in this group.
+         */
+        int visibleClientGroupItem();
+        /**
+         * Switch the currently visible client to the one at list index \p index.
+         */
+        void setVisibleClientGroupItem( int index );
+        /**
+         * Move the client at index \p index to the position before the client at index \p before.
+         */
+        void moveItemInClientGroup( int index, int before );
+        /**
+         * Move the client that's unique identifier is \p itemId to the position before the client
+         * at index \p before if set, otherwise the end of the list. This call is to move clients
+         * between two different groups, if moving in the same group then use
+         * moveItemInClientGroup() instead.
+         * \see itemId()
+         */
+        void moveItemToClientGroup( long itemId, int before = -1 );
+        /**
+         * Remove the client at index \p index from the group. If \p newGeom is set then the client
+         * will move and resize to the specified geometry, otherwise it will stay where the group
+         * is located.
+         */
+        void removeFromClientGroup( int index, const QRect& newGeom = QRect() );
+        /**
+         * Close the client at index \p index.
+         */
+        void closeClientGroupItem( int index );
+        /**
+         * Close all windows in this group.
+         */
+        void closeAllInClientGroup();
+        /**
+         * Display the right-click client menu belonging to the client at index \p index at the
+         * global coordinates specified by \p pos.
+         */
+        void displayClientMenu( int index, const QPoint& pos );
+        /**
+         * Determine which action the user has mapped \p button to. Useful for determining whether
+         * a button press was for window tab dragging or for displaying the client menu.
+         */
+        WindowOperation buttonToWindowOperation( Qt::MouseButtons button );
     };
 
 inline
