@@ -247,6 +247,16 @@ void Effect::postPaintWindow( EffectWindow* w )
     effects->postPaintWindow( w );
     }
 
+void Effect::paintEffectFrame( KWin::EffectFrame* frame, QRegion region, double opacity, double frameOpacity )
+    {
+    effects->paintEffectFrame( frame, region, opacity, frameOpacity );
+    }
+    
+bool Effect::provides( Feature ) 
+    { 
+    return false; 
+    }
+
 void Effect::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data )
     {
     effects->drawWindow( w, mask, region, data );
@@ -304,6 +314,10 @@ double Effect::animationTime( int defaultTime )
     return qMax( defaultTime * effects->animationTimeFactor(), 1. );
     }
 
+void Effect::numberDesktopsChanged( int )
+    {
+    }
+
 //****************************************
 // EffectsHandler
 //****************************************
@@ -382,112 +396,44 @@ KConfigGroup EffectsHandler::effectConfig( const QString& effectname )
     return kwinconfig->group( "Effect-" + effectname );
     }
 
-bool EffectsHandler::paintText( const QString& text, const QRect& rect, const QColor& color,
-        const QFont& font, const Qt::Alignment& alignment )
-{
-    QPainter p;
-    // Calculate size of the text
-    QFontMetrics fm( font );
-    QString painttext = fm.elidedText( text, Qt::ElideRight, rect.width() );
-    QRect textrect = fm.boundingRect( painttext );
-
-    // Create temporary QImage where the text will be drawn onto
-    QImage textImage( textrect.width(), textrect.height(), QImage::Format_ARGB32 );
-    textImage.fill( Qt::transparent );
-
-    // Draw the text
-    p.begin( &textImage );
-    p.setFont( font );
-    p.setRenderHint( QPainter::TextAntialiasing );
-    p.setPen( color );
-    p.drawText( -textrect.topLeft(), painttext );
-    p.end();
-
-    // Area covered by text
-    int rectX, rectY;
-    if( alignment & Qt::AlignLeft )
-        rectX = rect.x();
-    else if( alignment & Qt::AlignRight )
-        rectX = rect.right() - textrect.width();
-    else
-        rectX = rect.center().x() - textrect.width() / 2;
-    if( alignment & Qt::AlignTop )
-        rectY = rect.y();
-    else if( alignment & Qt::AlignBottom )
-        rectY = rect.bottom() - textrect.height();
-    else
-        rectY = rect.center().y() - textrect.height() / 2;
-    QRect area( rectX, rectY, textrect.width(), textrect.height() );
-
+bool EffectsHandler::checkDriverBlacklist( const KConfigGroup& blacklist )
+    {
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
-        GLTexture textTexture( textImage, GL_TEXTURE_RECTANGLE_ARB );
-        glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
-        glEnable( GL_BLEND );
-        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
-        textTexture.bind();
-        textTexture.render( infiniteRegion(), area );
-        textTexture.unbind();
-        glPopAttrib();
-        return true;
+        QString vendor   = QString((const char*)glGetString( GL_VENDOR ));
+        QString renderer = QString((const char*)glGetString( GL_RENDERER ));
+        QString version  = QString((const char*)glGetString( GL_VERSION ));
+        foreach( const QString& key, blacklist.keyList() )
+            {
+            // the key is a word in the renderer string or vendor referrencing the vendor in case of mesa
+            // e.g. "Intel" or "Ati"
+            if( renderer.contains( key, Qt::CaseInsensitive ) || vendor.contains( key, Qt::CaseInsensitive ) )
+                {
+                // the value for current key contains a string list of driver versions which have to be blacklisted
+                QStringList versions = blacklist.readEntry< QStringList >( key, QStringList() );
+                foreach( const QString& entry, versions )
+                    {
+                    QStringList parts = entry.split( ":-:" );
+                    if( parts.size() != 2 )
+                        {
+                        continue;
+                        }
+                    if( renderer.contains(parts[0], Qt::CaseInsensitive) &&
+                        version.contains(parts[1], Qt::CaseInsensitive) )
+                        {
+                        // the version matches the renderer string - this driver is blacklisted, return
+                        return true;
+                        }
+                    }
+                }
+            }
         }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-        {
-        XRenderPicture textPicture( QPixmap::fromImage( textImage ));
-        XRenderComposite( display(), textImage.depth() == 32 ? PictOpOver : PictOpSrc,
-            textPicture, None, effects->xrenderBufferPicture(),
-            0, 0, 0, 0, area.x(), area.y(), area.width(), area.height());
-        return true;
-        }
-#endif
     return false;
-}
-
-bool EffectsHandler::paintTextWithBackground( const QString& text, const QRect& rect, const QColor& color,
-        const QColor& bgcolor, const QFont& font, const Qt::Alignment& alignment )
-{
-    // Calculate size of the text
-    QFontMetrics fm( font );
-    QString painttext = fm.elidedText( text, Qt::ElideRight, rect.width() );
-    QRect textrect = fm.boundingRect( painttext );
-
-    // Area covered by text
-    int rectX, rectY;
-    if( alignment & Qt::AlignLeft )
-        rectX = rect.x();
-    else if( alignment & Qt::AlignRight )
-        rectX = rect.right() - textrect.width();
-    else
-        rectX = rect.center().x() - textrect.width() / 2;
-    if( alignment & Qt::AlignTop )
-        rectY = rect.y();
-    else if( alignment & Qt::AlignBottom )
-        rectY = rect.bottom() - textrect.height();
-    else
-        rectY = rect.center().y() - textrect.height() / 2;
-    QRect area( rectX, rectY, textrect.width(), textrect.height() );
-
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( effects->compositingType() == OpenGLCompositing )
-    {
-        glColor4f( bgcolor.redF(), bgcolor.greenF(), bgcolor.blueF(), bgcolor.alphaF() );
-        renderRoundBox( area.adjusted( -8, -3, 8, 3 ), 5 );
-
-        return paintText( text, rect, color, font, alignment );
-    }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-    {
-        xRenderRoundBox( effects->xrenderBufferPicture(), area.adjusted( -8, -3, 8, 3 ), 5, bgcolor );
-        return paintText( text, rect, color, font, alignment );
-    }
-#endif
+#else
     return false;
-}
+#endif
+    }
 
 
 EffectsHandler* effects = 0;
@@ -903,7 +849,8 @@ PaintClipper::Iterator::Iterator()
     if( clip() && effects->compositingType() == OpenGLCompositing )
         {
         glPushAttrib( GL_SCISSOR_BIT );
-        glEnable( GL_SCISSOR_TEST );
+        if( !effects->isRenderTargetBound() )
+            glEnable( GL_SCISSOR_TEST );
         data->rects = paintArea().rects();
         data->index = -1;
         next(); // move to the first one
@@ -951,7 +898,7 @@ void PaintClipper::Iterator::next()
     {
     data->index++;
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( clip() && effects->compositingType() == OpenGLCompositing && data->index < data->rects.count())
+    if( clip() && effects->compositingType() == OpenGLCompositing && !effects->isRenderTargetBound() && data->index < data->rects.count())
         {
         const QRect& r = data->rects[ data->index ];
         // Scissor rect has to be given in OpenGL coords
@@ -1316,6 +1263,13 @@ QRectF WindowMotionManager::transformedGeometry( EffectWindow *w ) const
     return geometry;
     }
 
+void WindowMotionManager::setTransformedGeometry( EffectWindow *w, const QRectF &geometry )
+    {
+    m_managedWindows[ w ].translation.setValue( geometry.topLeft() );
+    m_managedWindows[ w ].scale.setValue( QPointF( geometry.width()/qreal(w->width()),
+                                                   geometry.height()/qreal(w->height())));
+    }
+
 QRectF WindowMotionManager::targetGeometry( EffectWindow *w ) const
     {
     QRectF geometry( w->geometry() );
@@ -1343,334 +1297,60 @@ EffectWindow* WindowMotionManager::windowAtPoint( QPoint point, bool useStacking
     return NULL;
     }
 
+/***************************************************************
+ EffectFramePrivate
+***************************************************************/
+class EffectFramePrivate
+    {
+    public:
+        EffectFramePrivate();
+        ~EffectFramePrivate();
+
+        bool crossFading;
+        qreal crossFadeProgress;
+    };
+
+EffectFramePrivate::EffectFramePrivate()
+    : crossFading( false )
+    , crossFadeProgress( 1.0 )
+    {
+    }
+
+EffectFramePrivate::~EffectFramePrivate()
+    {
+    }
+
 /***************************************************************
  EffectFrame
 ***************************************************************/
-
-GLTexture* EffectFrame::m_unstyledTexture = NULL;
-
-EffectFrame::EffectFrame( Style style, bool staticSize, QPoint position, Qt::Alignment alignment )
-    : QObject()
-    , m_style( style )
-    , m_static( staticSize )
-    , m_point( position )
-    , m_alignment( alignment )
+EffectFrame::EffectFrame()
+    : d( new EffectFramePrivate )
     {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    m_texture = NULL;
-    m_textTexture = NULL;
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    m_picture = NULL;
-    m_textPicture = NULL;
-#endif
-
-    if( m_style == Unstyled )
-        {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-        if( effects->compositingType() == OpenGLCompositing && !m_unstyledTexture )
-            updateUnstyledTexture();
-#endif
-        }
-    else if( m_style == Styled )
-        {
-        m_frame.setImagePath( "widgets/background" );
-        m_frame.setCacheAllRenderedFrames( true );
-        connect( Plasma::Theme::defaultTheme(), SIGNAL( themeChanged() ), this, SLOT( plasmaThemeChanged() ));
-        }
     }
 
 EffectFrame::~EffectFrame()
     {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_texture;
-    delete m_textTexture;
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    delete m_picture;
-    delete m_textPicture;
-#endif
+    delete d;
     }
 
-void EffectFrame::free()
+qreal EffectFrame::crossFadeProgress() const
     {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_texture;
-    m_texture = NULL;
-    delete m_textTexture;
-    m_textTexture = NULL;
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    delete m_picture;
-    m_picture = NULL;
-    delete m_textPicture;
-    m_textPicture = NULL;
-#endif
+    return d->crossFadeProgress;
     }
 
-void EffectFrame::render( QRegion region, double opacity, double frameOpacity )
+void EffectFrame::setCrossFadeProgress( qreal progress )
     {
-    if( m_geometry.isEmpty() )
-        return; // Nothing to display
-
-    region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
-
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( effects->compositingType() == OpenGLCompositing )
-        {
-        glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
-        glEnable( GL_BLEND );
-        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
-        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
-
-        glPushMatrix();
-
-        // Render the actual frame
-        if( m_style == Unstyled )
-            {
-            const QRect& area = m_geometry.adjusted( -5, -5, 5, 5 );
-            const int roundness = 5;
-            QVector<float> verts, texCoords;
-            verts.reserve( 80 );
-            texCoords.reserve( 80 );
-
-            // Center
-            addQuadVertices( verts, area.left() + roundness, area.top() + roundness,
-                area.right() - roundness, area.bottom() - roundness );
-            addQuadVertices( texCoords, 0.5, 0.5, 0.5, 0.5 );
-
-            // Left
-            addQuadVertices( verts, area.left(), area.top() + roundness,
-                area.left() + roundness, area.bottom() - roundness );
-            addQuadVertices( texCoords, 0.0, 0.5, 0.5, 0.5 );
-            // Top
-            addQuadVertices( verts, area.left() + roundness, area.top(),
-                area.right() - roundness, area.top() + roundness );
-            addQuadVertices( texCoords, 0.5, 0.0, 0.5, 0.5 );
-            // Right
-            addQuadVertices( verts, area.right() - roundness, area.top() + roundness,
-                area.right(), area.bottom() - roundness );
-            addQuadVertices( texCoords, 0.5, 0.5, 1.0, 0.5 );
-            // Bottom
-            addQuadVertices( verts, area.left() + roundness, area.bottom() - roundness,
-                area.right() - roundness, area.bottom() );
-            addQuadVertices( texCoords, 0.5, 0.5, 0.5, 1.0 );
-
-            // Top-left
-            addQuadVertices( verts, area.left(), area.top(),
-                area.left() + roundness, area.top() + roundness );
-            addQuadVertices( texCoords, 0.0, 0.0, 0.5, 0.5 );
-            // Top-right
-            addQuadVertices( verts, area.right() - roundness, area.top(),
-                area.right(), area.top() + roundness );
-            addQuadVertices( texCoords, 0.5, 0.0, 1.0, 0.5 );
-            // Bottom-left
-            addQuadVertices( verts, area.left(), area.bottom() - roundness,
-                area.left() + roundness, area.bottom() );
-            addQuadVertices( texCoords, 0.0, 0.5, 0.5, 1.0 );
-            // Bottom-right
-            addQuadVertices( verts, area.right() - roundness, area.bottom() - roundness,
-                area.right(), area.bottom() );
-            addQuadVertices( texCoords, 0.5, 0.5, 1.0, 1.0 );
-
-            glColor4f( 0.0, 0.0, 0.0, opacity * frameOpacity );
-
-            m_unstyledTexture->bind();
-            m_unstyledTexture->enableNormalizedTexCoords();
-            renderGLGeometry( verts.count() / 2, verts.data(), texCoords.data() );
-            m_unstyledTexture->disableNormalizedTexCoords();
-            m_unstyledTexture->unbind();
-            }
-        else if( m_style == Styled )
-            {
-            if( !m_texture ) // Lazy creation
-                updateTexture();
-
-            glColor4f( 1.0, 1.0, 1.0, opacity * frameOpacity );
-
-            m_texture->bind();
-            qreal left, top, right, bottom;
-            m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
-            m_texture->render( region, m_geometry.adjusted( -left, -top, right, bottom ));
-            m_texture->unbind();
-            }
-
-        glColor4f( 1.0, 1.0, 1.0, opacity );
-
-        // Render icon
-        if( !m_icon.isNull() && !m_iconSize.isEmpty() )
-            {
-            QPoint topLeft( m_geometry.x(), m_geometry.center().y() - m_iconSize.height() / 2 );
-
-            GLTexture* icon = new GLTexture( m_icon ); // TODO: Cache
-            icon->bind();
-            icon->render( region, QRect( topLeft, m_iconSize ));
-            icon->unbind();
-            delete icon;
-            }
-
-        // Render text
-        if( !m_text.isEmpty() )
-            {
-            if( !m_textTexture ) // Lazy creation
-                updateTextTexture();
-            m_textTexture->bind();
-            m_textTexture->render( region, m_geometry );
-            m_textTexture->unbind();
-            }
-
-        glPopMatrix();
-        glPopAttrib();
-        }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-        {
-        // Render the actual frame
-        if( m_style == Unstyled )
-            xRenderRoundBox( effects->xrenderBufferPicture(), m_geometry.adjusted( -5, -5, 5, 5 ),
-                5, QColor( 0, 0, 0, int( opacity * frameOpacity * 255 )));
-        else if( m_style == Styled )
-            {
-            if( !m_picture ) // Lazy creation
-                updatePicture();
-            qreal left, top, right, bottom;
-            m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
-            QRect geom = m_geometry.adjusted( -left, -top, right, bottom );
-            XRenderComposite( display(), PictOpOver, *m_picture, None, effects->xrenderBufferPicture(),
-                0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
-            }
-
-        XRenderPicture fill = xRenderBlendPicture(opacity);
-        
-        // Render icon
-        if( !m_icon.isNull() && !m_iconSize.isEmpty() )
-            {
-            QPoint topLeft( m_geometry.x(), m_geometry.center().y() - m_iconSize.height() / 2 );
-
-            XRenderPicture* icon = new XRenderPicture( m_icon ); // TODO: Cache
-            QRect geom = QRect( topLeft, m_iconSize );
-            XRenderComposite( display(), PictOpOver, *icon, fill, effects->xrenderBufferPicture(),
-                0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
-            delete icon;
-            }
-
-        // Render text
-        if( !m_text.isEmpty() )
-            {
-            if( !m_textPicture ) // Lazy creation
-                updateTextPicture();
-            XRenderComposite( display(), PictOpOver, *m_textPicture, fill, effects->xrenderBufferPicture(),
-                0, 0, 0, 0, m_geometry.x(), m_geometry.y(), m_geometry.width(), m_geometry.height() );
-            }
-        }
-#endif
+    d->crossFadeProgress = progress;
     }
 
-void EffectFrame::setPosition( const QPoint& point )
+bool EffectFrame::isCrossFade() const
     {
-    m_point = point;
-    autoResize();
+    return d->crossFading;
     }
 
-void EffectFrame::setGeometry( const QRect& geometry, bool force )
+void EffectFrame::enableCrossFade( bool enable )
     {
-    QRect oldGeom = m_geometry;
-    m_geometry = geometry;
-    if( m_geometry == oldGeom && !force )
-        return;
-    effects->addRepaint( oldGeom );
-    effects->addRepaint( m_geometry );
-    if( m_geometry.size() == oldGeom.size() && !force )
-        return;
-
-    if( m_style == Styled )
-        {
-        qreal left, top, right, bottom;
-        m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
-        m_frame.resizeFrame( m_geometry.adjusted( -left, -top, right, bottom ).size() );
-        }
-
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( effects->compositingType() == OpenGLCompositing )
-        {
-        delete m_texture;
-        m_texture = NULL;
-        delete m_textTexture;
-        m_textTexture = NULL;
-        }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-        {
-        delete m_picture;
-        m_picture = NULL;
-        delete m_textPicture;
-        m_textPicture = NULL;
-        }
-#endif
-    }
-
-void EffectFrame::setText( const QString& text )
-    {
-    m_text = text;
-    QRect oldGeom = m_geometry;
-    autoResize();
-    if( oldGeom == m_geometry )
-        { // Wasn't updated in autoResize()
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-        if( effects->compositingType() == OpenGLCompositing )
-            {
-            delete m_textTexture;
-            m_textTexture = NULL;
-            }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-        if( effects->compositingType() == XRenderCompositing )
-            {
-            delete m_textPicture;
-            m_textPicture = NULL;
-            }
-#endif
-        }
-    }
-
-void EffectFrame::setFont( const QFont& font )
-    {
-    m_font = font;
-    QRect oldGeom = m_geometry;
-    if( !m_text.isEmpty() )
-        autoResize();
-    if( oldGeom == m_geometry )
-        { // Wasn't updated in autoResize()
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-        if( effects->compositingType() == OpenGLCompositing )
-            {
-            delete m_textTexture;
-            m_textTexture = NULL;
-            }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-        if( effects->compositingType() == XRenderCompositing )
-            {
-            delete m_textPicture;
-            m_textPicture = NULL;
-            }
-#endif
-        }
-    }
-
-void EffectFrame::setIcon( const QPixmap& icon )
-    {
-    m_icon = icon;
-    if( m_iconSize.isEmpty() ) // Set a size if we don't already have one
-        setIconSize( m_icon.size() );
-    }
-
-void EffectFrame::setIconSize( const QSize& size )
-    {
-    m_iconSize = size;
-    autoResize();
+    d->crossFading = enable;
     }
 
 QColor EffectFrame::styledTextColor()
@@ -1678,178 +1358,4 @@ QColor EffectFrame::styledTextColor()
     return Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor );
     }
 
-void EffectFrame::plasmaThemeChanged()
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( effects->compositingType() == OpenGLCompositing )
-        {
-        delete m_texture;
-        m_texture = NULL;
-        delete m_textTexture;
-        m_textTexture = NULL;
-        }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-        {
-        delete m_picture;
-        m_picture = NULL;
-        delete m_textPicture;
-        m_textPicture = NULL;
-        }
-#endif
-    }
-
-void EffectFrame::autoResize()
-    {
-    if( m_static )
-        return; // Not automatically resizing
-
-    QRect geometry;
-
-    // Set size
-    if( !m_text.isEmpty() )
-        {
-        QFontMetrics metrics( m_font );
-        geometry.setSize( metrics.size( 0, m_text ));
-        }
-    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
-        {
-        geometry.setLeft( -m_iconSize.width() );
-        if( m_iconSize.height() > geometry.height() )
-            geometry.setHeight( m_iconSize.height() );
-        }
-
-    // Set position
-    if( m_alignment & Qt::AlignLeft )
-        geometry.moveLeft( m_point.x() );
-    else if( m_alignment & Qt::AlignRight )
-        geometry.moveLeft( m_point.x() - geometry.width() );
-    else
-        geometry.moveLeft( m_point.x() - geometry.width() / 2 );
-    if( m_alignment & Qt::AlignTop )
-        geometry.moveTop( m_point.y() );
-    else if( m_alignment & Qt::AlignBottom )
-        geometry.moveTop( m_point.y() - geometry.height() );
-    else
-        geometry.moveTop( m_point.y() - geometry.height() / 2 );
-
-    setGeometry( geometry );
-    }
-
-void EffectFrame::updateTexture()
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_texture;
-    if( m_style == Styled )
-        m_texture = new GLTexture( m_frame.framePixmap() );
-#endif
-    }
-
-void EffectFrame::updateTextTexture()
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_textTexture;
-
-    if( m_text.isEmpty() )
-        return;
-
-    // Determine position on texture to paint text
-    QRect rect( QPoint( 0, 0 ), m_geometry.size() );
-    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
-        rect.setLeft( m_iconSize.width() );
-
-    // If static size elide text as required
-    QString text = m_text;
-    if( m_static )
-        {
-        QFontMetrics metrics( m_font );
-        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
-        }
-
-    QImage image( m_geometry.size(), QImage::Format_ARGB32 );
-    image.fill( Qt::transparent );
-    QPainter p( &image );
-    p.setFont( m_font );
-    if( m_style == Styled )
-        p.setPen( styledTextColor() );
-    else // TODO: What about no frame? Custom color setting required
-        p.setPen( Qt::white );
-    p.drawText( rect, m_alignment, text );
-    p.end();
-    m_textTexture = new GLTexture( image );
-#endif
-    }
-
-void EffectFrame::updatePicture()
-    {
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    delete m_picture;
-    if( m_style == Styled )
-        m_picture = new XRenderPicture( m_frame.framePixmap() );
-#endif
-    }
-
-void EffectFrame::updateTextPicture()
-    { // Mostly copied from EffectFrame::updateTextTexture() above
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    delete m_textPicture;
-
-    if( m_text.isEmpty() )
-        return;
-
-    // Determine position on texture to paint text
-    QRect rect( QPoint( 0, 0 ), m_geometry.size() );
-    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
-        rect.setLeft( m_iconSize.width() );
-
-    // If static size elide text as required
-    QString text = m_text;
-    if( m_static )
-        {
-        QFontMetrics metrics( m_font );
-        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
-        }
-
-    QPixmap pixmap( m_geometry.size() );
-    pixmap.fill( Qt::transparent );
-    QPainter p( &pixmap );
-    p.setFont( m_font );
-    if( m_style == Styled )
-        p.setPen( styledTextColor() );
-    else // TODO: What about no frame? Custom color setting required
-        p.setPen( Qt::white );
-    p.drawText( rect, m_alignment, text );
-    p.end();
-    m_textPicture = new XRenderPicture( pixmap );
-#endif
-    }
-
-void EffectFrame::cleanup()
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_unstyledTexture;
-    m_unstyledTexture = NULL;
-#endif
-    }
-
-void EffectFrame::updateUnstyledTexture()
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    delete m_unstyledTexture;
-    // Based off circle() from kwinxrenderutils.cpp
-#define CS 8
-    QImage tmp( 2 * CS, 2 * CS, QImage::Format_ARGB32 );
-    tmp.fill( Qt::transparent );
-    QPainter p( &tmp );
-    p.setRenderHint( QPainter::Antialiasing );
-    p.setPen( Qt::NoPen );
-    p.setBrush( Qt::black );
-    p.drawEllipse( tmp.rect() );
-    p.end();
-#undef CS
-    m_unstyledTexture = new GLTexture( tmp );
-#endif
-    }
-
 } // namespace
