@@ -25,18 +25,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QPalette>
 #include <QPixmap>
+#include <QProcess>
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
+#include <klocale.h>
 
 #include <QDesktopWidget>
 
 #include "client.h"
 #include "compositingprefs.h"
+#include "lib/kwinglplatform.h"
 
 #include <kephal/screens.h>
 
+#ifdef HAVE_XRANDR
+#include <X11/extensions/Xrandr.h>
+#endif
+
+
 #endif
 
 namespace KWin
@@ -44,6 +52,47 @@ namespace KWin
 
 #ifndef KCMRULES
 
+int currentRefreshRate()
+    {
+    int rate = -1;
+    if( options->refreshRate > 0 ) // use manually configured refresh rate
+        rate = options->refreshRate;
+    else if ( GLPlatform::instance()->driver() == Driver_NVidia )
+        {
+        QProcess nvidia_settings;
+        nvidia_settings.start( "nvidia-settings", QStringList() << "-t" << "-q" << "RefreshRate", QIODevice::ReadOnly );
+        nvidia_settings.waitForFinished();
+        if ( nvidia_settings.exitStatus() == QProcess::NormalExit )
+            {
+            QString reply = QString::fromLocal8Bit( nvidia_settings.readAllStandardOutput() );
+            bool ok;
+            const float frate = reply.split(' ').first().toFloat( &ok );
+            if ( !ok )
+                rate = -1;
+            else
+                rate = qRound(frate);
+            }
+        }
+#ifdef HAVE_XRANDR
+    else if( Extensions::randrAvailable() )
+        {
+        XRRScreenConfiguration *config = XRRGetScreenInfo( display(), rootWindow() );
+        rate = XRRConfigCurrentRate( config );
+        XRRFreeScreenConfigInfo( config );
+        }
+#endif    
+        
+    // 0Hz or less is invalid, so we fallback to a default rate
+    if( rate <= 0 )
+        rate = 50;
+    // QTimer gives us 1msec (1000Hz) at best, so we ignore anything higher;
+    // however, additional throttling prevents very high rates from taking place anyway
+    else if( rate > 1000 )
+        rate = 1000;
+    kDebug( 1212 ) << "Refresh rate " << rate << "Hz";
+    return rate;
+    }
+
 Options::Options()
     : electric_borders( 0 )
     , electric_border_delay( 0 )
@@ -200,6 +249,7 @@ unsigned long Options::updateSettings()
     CmdAllWheel = mouseWheelCommand(config.readEntry("CommandAllWheel","Nothing"));
 
     config=KConfigGroup(_config,"Compositing");
+    maxFpsInterval = qRound(1000.0/config.readEntry( "MaxFPS", 60 ));
     refreshRate = config.readEntry( "RefreshRate", 0 );
 
     // Read button tooltip animation effect from kdeglobals
@@ -248,6 +298,7 @@ void Options::reloadCompositingSettings()
     // Compositing settings
     CompositingPrefs prefs;
     prefs.detect();
+
     useCompositing = config.readEntry( "Enabled" , prefs.recommendCompositing());
 
     QString compositingBackend = config.readEntry("Backend", "OpenGL");
@@ -265,7 +316,7 @@ void Options::reloadCompositingSettings()
         glMode = GLFallback;
     glDirect = config.readEntry("GLDirect", prefs.enableDirectRendering() );
     glVSync = config.readEntry("GLVSync", prefs.enableVSync() );
-    glSmoothScale = qBound( -1, config.readEntry( "GLTextureFilter", -1 ), 2 );
+    glSmoothScale = qBound( -1, config.readEntry( "GLTextureFilter", 2 ), 2 );
     glStrictBinding = config.readEntry( "GLStrictBinding", prefs.strictBinding());
 
     xrenderSmoothScale = config.readEntry("XRenderSmoothScale", false );
