@@ -1,7 +1,4 @@
-/*****************************************************************
-
-Copyright 2008 Christian Mollekopf <chrigi_1@hotmail.com>
-
+/***************************************************************** Copyright 2008 Christian Mollekopf <chrigi_1@hotmail.com>
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
@@ -24,7 +21,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "programgroupingstrategy.h"
 
 #include <QAction>
-#include <QPointer>
+#include <QWeakPointer>
 
 #include <KDebug>
 #include <KLocale>
@@ -46,7 +43,7 @@ public:
     }
 
     AbstractGroupingStrategy::EditableGroupProperties editableGroupProperties;
-    QPointer<AbstractGroupableItem> tempItem;
+    QWeakPointer<AbstractGroupableItem> tempItem;
     QStringList blackList; //Programs in this list should not be grouped
 };
 
@@ -72,9 +69,9 @@ QList<QAction*> ProgramGroupingStrategy::strategyActions(QObject *parent, Abstra
     QAction *a = new QAction(parent);
     QString name = className(item);
     if (d->blackList.contains(name)) {
-        a->setText(i18n("Allow This Program to Be Grouped"));
+        a->setText(i18n("Allow this program to be grouped"));
     } else {
-        a->setText(i18n("Do Not Allow This Program to Be Grouped"));
+        a->setText(i18n("Do not allow this program to be grouped"));
     }
     connect(a, SIGNAL(triggered()), this, SLOT(toggleGrouping()));
 
@@ -87,42 +84,53 @@ QList<QAction*> ProgramGroupingStrategy::strategyActions(QObject *parent, Abstra
 QString ProgramGroupingStrategy::className(AbstractGroupableItem *item)
 {
     QString name;
-    if (item->isGroupItem()) { //maybe add the condition that the subgroup was created by programGrouping
+    if (item->itemType() == GroupItemType) { //maybe add the condition that the subgroup was created by programGrouping
         TaskGroup *group = qobject_cast<TaskGroup*>(item);
         TaskItem *task = qobject_cast<TaskItem*>(group->members().first()); //There are only TaskItems in programGrouping groups
         return task->task()->classClass();
     }
 
-    return (qobject_cast<TaskItem*>(item))->task()->classClass();
+        return (qobject_cast<TaskItem*>(item))->task()->classClass();
 }
 
+
 void ProgramGroupingStrategy::toggleGrouping()
 {
-    if (!d->tempItem) {
+    AbstractGroupableItem *tempItem = d->tempItem.data();
+    if (!tempItem) {
         return;
     }
 
-    QString name = className(d->tempItem);
+    QString name = className(tempItem);
 
     if (d->blackList.contains(name)) {
         d->blackList.removeAll(name);
-        if (d->tempItem->isGroupItem()) {
-            foreach (AbstractGroupableItem *item, (qobject_cast<TaskGroup*>(d->tempItem))->members()) {
+        if (tempItem->itemType() == GroupItemType) {
+            foreach (AbstractGroupableItem *item, (qobject_cast<TaskGroup*>(tempItem))->members()) {
                 handleItem(item);
             }
         } else {
-            handleItem(d->tempItem);
+            handleItem(tempItem);
         }
     } else {
         d->blackList.append(name);
-        if (d->tempItem->isGroupItem()) {
-            closeGroup(qobject_cast<TaskGroup*>(d->tempItem));
-        } else if (rootGroup()) {
-            rootGroup()->add(d->tempItem);
+        GroupPtr root = rootGroup();
+        if (tempItem->itemType() == GroupItemType) {
+            closeGroup(qobject_cast<TaskGroup*>(tempItem));
+        } else if (root) {
+            root->add(tempItem);
+        }
+
+        if (root) {
+            foreach (AbstractGroupableItem *item, root->members()) {
+                if (item->itemType() == GroupItemType) {
+                    untoggleGroupingOn(static_cast<TaskGroup*>(item), name);
+                }
+            }
         }
     }
 
-    d->tempItem = 0;
+    d->tempItem.clear();
 
     // Save immediately. Much better than saving in the destructor,
     // since this class is deleted at every change of virtual desktop (!)
@@ -134,6 +142,31 @@ void ProgramGroupingStrategy::toggleGrouping()
     blackGroup.config()->sync();
 }
 
+void ProgramGroupingStrategy::untoggleGroupingOn(TaskGroup *group, const QString &name)
+{
+    if (!group->parentGroup()) {
+        return;
+    }
+
+    foreach (AbstractGroupableItem *item, group->members()) {
+        if (item->itemType() == GroupItemType) {
+            untoggleGroupingOn(static_cast<TaskGroup*>(item), name);
+        }
+    }
+
+    foreach (AbstractGroupableItem *item, group->members()) {
+        if (!item->itemType() == GroupItemType && name == static_cast<TaskItem *>(item)->task()->classClass())  {
+            if (group->parentGroup()) {
+                group->parentGroup()->add(item);
+            }
+        }
+    }
+
+    if (group->members().isEmpty()) {
+        closeGroup(group);
+    }
+}
+
 void ProgramGroupingStrategy::handleItem(AbstractGroupableItem *item)
 {
     GroupPtr root = rootGroup();
@@ -142,11 +175,11 @@ void ProgramGroupingStrategy::handleItem(AbstractGroupableItem *item)
         return;
     }
 
-    if (item->isGroupItem()) {
+    if (item->itemType() == GroupItemType) {
         //kDebug() << "item is groupitem";
         root->add(item);
         return;
-    } else if (d->blackList.contains((qobject_cast<TaskItem*>(item))->task()->classClass())) {
+    } else if (d->blackList.contains((static_cast<TaskItem*>(item))->task()->classClass())) {
         //kDebug() << "item is in blacklist";
         root->add(item);
         return;
@@ -163,11 +196,11 @@ bool ProgramGroupingStrategy::programGrouping(TaskItem* taskItem, TaskGroup* gro
 {
     //kDebug();
     QList<AbstractGroupableItem *> list;
-    const QString name = taskItem->task()->classClass();
+    QString name = taskItem->task()->classClass();
 
     //search for an existing group
     foreach (AbstractGroupableItem *item, groupItem->members()) {
-        if (item->isGroupItem()) {
+        if (item->itemType() == GroupItemType) {
             //TODO: maybe add the condition that the subgroup was created by programGrouping?
             if (programGrouping(taskItem, static_cast<TaskGroup*>(item))) {
                 //kDebug() << "joined subGroup";
@@ -184,7 +217,7 @@ bool ProgramGroupingStrategy::programGrouping(TaskItem* taskItem, TaskGroup* gro
     if (!list.isEmpty()) {
         if (groupItem->isRootGroup()) {
             //kDebug() << "create Group root group";
-            QIcon icon = taskItem->task()->icon();
+            QIcon icon = taskItem->icon();
             list.append(taskItem);
             TaskGroup* group = createGroup(list);
             group->setName(name);
