@@ -33,6 +33,7 @@
 // KDE
 #include <KAuthorized>
 #include <KDebug>
+#include <KDesktopFile>
 
 #include <taskmanager/taskactions.h>
 #include <taskmanager/taskmanager.h>
@@ -50,6 +51,7 @@
 #include "tasks.h"
 #include "taskitemlayout.h"
 #include "windowtaskitem.h"
+#include "applauncheritem.h"
 
 TaskGroupItem::TaskGroupItem(QGraphicsWidget *parent, Tasks *applet)
     : AbstractTaskItem(parent, applet),
@@ -67,9 +69,11 @@ TaskGroupItem::TaskGroupItem(QGraphicsWidget *parent, Tasks *applet)
       m_collapsed(true),
       m_mainLayout(0),
       m_popupDialog(0),
-      m_popupLostFocus(false)
+      m_updateTimer(0),
+      m_changes(TaskManager::TaskUnchanged)
 {
     setAcceptDrops(true);
+    setFlag(ItemClipsChildrenToShape, true);
 }
 
 
@@ -201,12 +205,28 @@ void TaskGroupItem::updateTask(::TaskManager::TaskChanges changes)
         return;
     }
 
+    m_changes |= changes;
+
+    if (!m_updateTimer)  {
+        m_updateTimer = new QTimer(this);
+        m_updateTimer->setInterval(10);
+        m_updateTimer->setSingleShot(true);
+        connect(m_updateTimer, SIGNAL(timeout()), this, SLOT(checkUpdates()));
+    }
+
+    m_updateTimer->start();
+}
+
+void TaskGroupItem::checkUpdates()
+{
     bool needsUpdate = false;
     // task flags
     TaskFlags flags = m_flags;
     if (m_group.data()->isActive()) {
         flags |= TaskHasFocus;
-        emit activated(this);
+        if (!(m_flags & TaskHasFocus)) {
+            emit activated(this);
+        }
     } else {
         flags &= ~TaskHasFocus;
     }
@@ -229,22 +249,23 @@ void TaskGroupItem::updateTask(::TaskManager::TaskChanges changes)
     }
 
     // basic title and icon
-    if (changes & TaskManager::IconChanged) {
+    if (m_changes & TaskManager::IconChanged) {
         needsUpdate = true;
     }
 
-    if (changes & TaskManager::NameChanged) {
+    if (m_changes & TaskManager::NameChanged) {
         needsUpdate = true;
         textChanged();
     }
 
     if (Plasma::ToolTipManager::self()->isVisible(this) &&
-        (changes & TaskManager::IconChanged ||
-        changes & TaskManager::NameChanged ||
-        changes & TaskManager::DesktopChanged)) {
+        (m_changes & TaskManager::IconChanged ||
+         m_changes & TaskManager::NameChanged ||
+         m_changes & TaskManager::DesktopChanged)) {
         updateToolTip();
     }
 
+    m_changes = TaskManager::TaskUnchanged;
     if (needsUpdate) {
         //redraw
         queueUpdate();
@@ -292,21 +313,23 @@ void TaskGroupItem::updateToolTip()
 
 void TaskGroupItem::reload()
 {
-    QList <AbstractGroupableItem *> itemsToRemove = m_groupMembers.keys();
     if (!group()) {
         return;
     }
 
+    QHash<AbstractGroupableItem *, AbstractTaskItem*> itemsToRemove = m_groupMembers;
     foreach (AbstractGroupableItem *item, group()->members()) {
         if (!item) {
             kDebug() << "invalid Item";
             continue;
         }
 
-        itemsToRemove.removeAll(item);
+        if (itemsToRemove.contains(item)) {
+            itemsToRemove.insert(item, 0);
+        }
         itemAdded(item);
 
-        if (item->isGroupItem()) {
+        if (item->itemType() == TaskManager::GroupItemType) {
             TaskGroupItem *group = qobject_cast<TaskGroupItem *>(abstractTaskItem(item));
             if (group) {
                 group->reload();
@@ -314,12 +337,12 @@ void TaskGroupItem::reload()
         }
     }
 
-    foreach (AbstractGroupableItem * item, itemsToRemove) { //remove unused items
-        if (!item) {
-            kDebug() << "invalid Item";
-            continue;
+    QHashIterator<AbstractGroupableItem *, AbstractTaskItem*> it(itemsToRemove);
+    while (it.hasNext()) {
+        it.next();
+        if (it.key() && it.value()) {
+            itemRemoved(it.key());
         }
-        itemRemoved(item);
     }
 }
 
@@ -428,10 +451,13 @@ AbstractTaskItem *TaskGroupItem::createAbstractItem(TaskManager::AbstractGroupab
     //kDebug() << "item to create" << groupableItem << endl;
     AbstractTaskItem *item = 0;
 
-    if (groupableItem->isGroupItem()) {
+    if (groupableItem->itemType() == TaskManager::GroupItemType) {
         TaskGroupItem *groupItem = new TaskGroupItem(this, m_applet);
         groupItem->setGroup(static_cast<TaskManager::TaskGroup*>(groupableItem));
         item = groupItem;
+    } else if (groupableItem->itemType() == TaskManager::LauncherItemType) {
+        AppLauncherItem *launcherItem = new AppLauncherItem(this, m_applet, static_cast<TaskManager::LauncherItem*>(groupableItem));
+        item = launcherItem;
     } else { //it's a window task
         WindowTaskItem *windowItem = new WindowTaskItem(this, m_applet);
         windowItem->setTask(static_cast<TaskManager::TaskItem*>(groupableItem));
@@ -503,6 +529,8 @@ void TaskGroupItem::itemAdded(TaskManager::AbstractGroupableItem * groupableItem
             connect(item, SIGNAL(changed()), this, SLOT(relayoutItems()));
         }
     }
+
+    update();
 }
 
 void TaskGroupItem::itemRemoved(TaskManager::AbstractGroupableItem * groupableItem)
@@ -558,22 +586,22 @@ void TaskGroupItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
         return;
     }
 
-    if ((event->buttons() & Qt::LeftButton) && (event->modifiers() & Qt::ControlModifier)) {
-        QList<WId> ids;
-        foreach (AbstractGroupableItem *groupable, m_group.data()->members()) {
-            if (groupable->isGroupItem()) {
-                //TODO: recurse through sub-groups?
-            } else {
-                TaskItem * item = dynamic_cast<TaskItem*>(groupable);
-                if (item && item->task()) {
-                    ids << item->task()->info().win();
+    if (event->buttons() & Qt::LeftButton) {
+        if (event->modifiers() & Qt::ControlModifier) {
+            QList<WId> ids;
+            foreach (AbstractGroupableItem *groupable, m_group.data()->members()) {
+                if (groupable->itemType() == TaskManager::GroupItemType) {
+                    //TODO: recurse through sub-groups?
+                } else {
+                    TaskItem * item = dynamic_cast<TaskItem*>(groupable);
+                    if (item && item->task()) {
+                        ids << item->task()->info().win();
+                    }
                 }
             }
-        }
-        Plasma::WindowEffects::presentWindows(m_applet->view()->winId(), ids);
-    } else if ((event->buttons() & Qt::LeftButton) && !m_popupLostFocus) {
-        if (m_applet->groupManager().sortingStrategy() == TaskManager::GroupManager::ManualSorting ||
-            m_applet->groupManager().groupingStrategy() == TaskManager::GroupManager::ManualGrouping) {
+            Plasma::WindowEffects::presentWindows(m_applet->view()->winId(), ids);
+        } else if (m_applet->groupManager().sortingStrategy() == TaskManager::GroupManager::ManualSorting ||
+                   m_applet->groupManager().groupingStrategy() == TaskManager::GroupManager::ManualGrouping) {
             if (!m_popupMenuTimer) {
                 m_popupMenuTimer = new QTimer(this);
                 m_popupMenuTimer->setSingleShot(true);
@@ -652,12 +680,11 @@ void TaskGroupItem::popupMenu()
         m_popupDialog = new Plasma::Dialog();
         KWindowSystem::setType(m_popupDialog->winId(), NET::PopupMenu);
         m_popupDialog->setAttribute(Qt::WA_X11NetWmWindowTypeDock);
+        connect(m_popupDialog, SIGNAL(dialogVisible(bool)), this, SLOT(popupVisibilityChanged(bool)));
         connect(m_popupDialog, SIGNAL(dialogVisible(bool)), m_applet, SLOT(setPopupDialog(bool)));
         connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(handleActiveWindowChanged(WId)));
         KWindowSystem::setState(m_popupDialog->winId(), NET::SkipTaskbar| NET::SkipPager);
-        m_popupDialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
-        //TODO in the future it may be possible to use the Qt::Popup flag instead of the eventFilter, but for now the focus works better with the eventFilter
-        m_popupDialog->installEventFilter(this);
+        m_popupDialog->setWindowFlags(Qt::Popup | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
 
         int left, top, right, bottom;
         m_popupDialog->getContentsMargins(&left, &top, &right, &bottom);
@@ -674,7 +701,6 @@ void TaskGroupItem::popupMenu()
         }
 
         QRect rect = iconGeometry();
-        publishIconGeometry(rect);
     } else {
         m_tasksLayout->setOrientation(Plasma::Vertical);
         m_tasksLayout->setMaximumRows(1);
@@ -696,23 +722,13 @@ void TaskGroupItem::popupMenu()
     }
 }
 
-bool TaskGroupItem::eventFilter(QObject *watched, QEvent *event)
+void TaskGroupItem::popupVisibilityChanged(bool visible)
 {
-    if (watched == m_popupDialog && event->type() == QEvent::WindowDeactivate) {
-        Q_ASSERT(m_popupDialog);
-        m_popupLostFocus = true; //avoid opening it again when clicking on the group
-        if (m_applet->location() != Plasma::Floating) {
-            m_popupDialog->animatedHide(Plasma::locationToInverseDirection(m_applet->location()));
-        } else {
-            m_popupDialog->hide();
-        }
-
+    if (!visible) {
         QRect rect = iconGeometry();
         publishIconGeometry(rect);
-        QTimer::singleShot(100, this, SLOT(clearPopupLostFocus()));
+        update();
     }
-
-    return QGraphicsWidget::eventFilter(watched, event);
 }
 
 bool TaskGroupItem::focusNextPrevChild(bool next)
@@ -769,11 +785,6 @@ bool TaskGroupItem::focusSubTask(bool next, bool activate)
     }
 }
 
-void TaskGroupItem::clearPopupLostFocus()
-{
-    m_popupLostFocus = false;
-}
-
 void TaskGroupItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
     if (QPoint(event->screenPos() - event->buttonDownScreenPos(Qt::LeftButton)).manhattanLength() < QApplication::startDragDistance()) {
@@ -990,7 +1001,7 @@ void  TaskGroupItem::itemPositionChanged(AbstractGroupableItem * item)
 
     Q_ASSERT(item);
 
-    if (item->isGroupItem()) {
+    if (item->itemType() == TaskManager::GroupItemType) {
         TaskGroupItem *groupItem = static_cast<TaskGroupItem*>(abstractTaskItem(item));
         if (groupItem) {
             groupItem->unsplitGroup();
@@ -1018,10 +1029,8 @@ void  TaskGroupItem::itemPositionChanged(AbstractGroupableItem * item)
 void TaskGroupItem::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
     //kDebug()<<"Drag enter";
-    if (collapsed() &&
-        (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
-         event->mimeData()->hasFormat(TaskManager::Task::groupMimetype()))) {
-        event->acceptProposedAction();
+    if (collapsed() && shouldIgnoreDragEvent(event)) {
+        event->ignore();
         //kDebug()<<"Drag enter accepted";
     } else {
         event->accept();
@@ -1091,6 +1100,28 @@ void TaskGroupItem::dropEvent(QGraphicsSceneDragDropEvent *event)
 
         //kDebug() << "TaskItemLayout dropEvent done";
         event->acceptProposedAction();
+    } else if (event->mimeData()->hasFormat("text/uri-list")) {
+        KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
+        foreach (const KUrl &url, urls) {
+            const bool exists = m_applet->groupManager().launcherExists(url);
+            if (exists) {
+                // it exists; if we are doing manual sorting, make sure it is in the right location if
+                // it is in this group .. otherwise, we can do nothing.
+                if (m_applet->groupManager().sortingStrategy() == TaskManager::GroupManager::ManualSorting) {
+                    QHashIterator<AbstractGroupableItem *, AbstractTaskItem*> it(m_groupMembers);
+                    while (it.hasNext()) {
+                        it.next();
+                        if (it.key()->itemType() == TaskManager::LauncherItemType &&
+                            static_cast<LauncherItem *>(it.key())->url() == url) {
+                            layoutTaskItem(it.value(), event->pos());
+                            break;
+                        }
+                    }
+                }
+            } else {
+                m_applet->groupManager().addLauncher(url);
+            }
+        }
     } else {
         event->ignore();
     }
@@ -1164,7 +1195,7 @@ void TaskGroupItem::handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraph
 
 void TaskGroupItem::layoutTaskItem(AbstractTaskItem* item, const QPointF &pos)
 {
-    if (!m_tasksLayout) {
+    if (!m_tasksLayout || !item->abstractItem()) {
         return;
     }
 
@@ -1203,38 +1234,40 @@ int TaskGroupItem::indexOf(AbstractTaskItem *task, bool descendGroups)
 
     foreach (AbstractGroupableItem *item, m_group.data()->members()) {
         AbstractTaskItem *taskItem = abstractTaskItem(item);
-        if (taskItem) {
-            if (task == taskItem) {
-                if (descendGroups) {
-                    TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
-                    if (groupItem) {
-                        int subIndex = groupItem->indexOf(groupItem->activeSubTask());
-                        if (subIndex == -1) {
-                            index += groupItem->count();
-                        } else {
-                            return index + subIndex;
-                        }
-                    }
-                }
-
-                return index;
-            }
+        if (!taskItem) {
+            continue;
+        }
 
+        if (task == taskItem) {
             if (descendGroups) {
                 TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
                 if (groupItem) {
-                    int subIndex = groupItem->indexOf(task);
+                    int subIndex = groupItem->indexOf(groupItem->activeSubTask());
                     if (subIndex == -1) {
                         index += groupItem->count();
                     } else {
-                        return index+subIndex;
+                        return index + subIndex;
                     }
+                }
+            }
+
+            return index;
+        }
+
+        if (descendGroups) {
+            TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
+            if (groupItem) {
+                int subIndex = groupItem->indexOf(task);
+                if (subIndex == -1) {
+                    index += groupItem->count();
                 } else {
-                    ++index;
+                    return index+subIndex;
                 }
             } else {
                 ++index;
             }
+        } else {
+            ++index;
         }
     }
     return -1;
@@ -1248,14 +1281,12 @@ AbstractTaskItem * TaskGroupItem::activeSubTask()
 
     foreach (AbstractGroupableItem *item, m_group.data()->members()) {
         AbstractTaskItem *taskItem = abstractTaskItem(item);
-        if (taskItem) {
-            if(taskItem->isActive()) {
-                TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
-                if (groupItem) {
-                    return groupItem->activeSubTask();
-                }
-                return taskItem;
+        if (taskItem && taskItem->isActive()) {
+            TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
+            if (groupItem) {
+                return groupItem->activeSubTask();
             }
+            return taskItem;
         }
     }
 
@@ -1345,17 +1376,23 @@ int TaskGroupItem::optimumCapacity()
 
 AbstractTaskItem* TaskGroupItem::abstractTaskItem(AbstractGroupableItem * item)
 {
-    AbstractTaskItem *abstractTaskItem = 0; 
-    if (m_groupMembers.contains(item)) {
-        abstractTaskItem = m_groupMembers.value(item);
-    } else {
-        foreach(AbstractTaskItem *taskItem, m_groupMembers) {
+    if (!item) {
+        return 0;
+    }
+
+    AbstractTaskItem *abstractTaskItem = m_groupMembers.value(item); 
+    if (!abstractTaskItem) {
+        foreach (AbstractTaskItem *taskItem, m_groupMembers) {
             TaskGroupItem *group = qobject_cast<TaskGroupItem*>(taskItem);
             if (group) {
                 abstractTaskItem = group->abstractTaskItem(item);
+                if (abstractTaskItem) {
+                    break;
+                }
             }
         }
     }
+
     //kDebug() << "item not found";
     return abstractTaskItem;
 }
