@@ -120,7 +120,8 @@ bool PowermanagementEngine::sourceRequestEvent(const QString &name)
         foreach (const Solid::Device &deviceBattery, listBattery) {
             const Solid::Battery* battery = deviceBattery.as<Solid::Battery>();
 
-            if (battery && battery->type() == Solid::Battery::PrimaryBattery) {
+            if (battery && (battery->type() == Solid::Battery::PrimaryBattery ||
+                            battery->type() == Solid::Battery::UpsBattery)) {
                 const QString source = QString("Battery%1").arg(index++);
 
                 batterySources << source;
@@ -247,7 +248,8 @@ void PowermanagementEngine::deviceAdded(const QString& udi)
     if (device.isValid()) {
         const Solid::Battery* battery = device.as<Solid::Battery>();
 
-        if (battery && battery->type() == Solid::Battery::PrimaryBattery) {
+        if (battery && (battery->type() == Solid::Battery::PrimaryBattery ||
+                        battery->type() == Solid::Battery::UpsBattery)) {
             int index = 0;
             QStringList sourceNames(m_batterySources.values());
             while (sourceNames.contains(QString("Battery%1").arg(index))) {
