@@ -24,7 +24,6 @@
 #include "runnersconfig.h"
 #include "../common/linearappletoverlay.h"
 #include "../common/appletmovespacer.h"
-#include "../common/nettoolbox.h"
 #include "iconactioncollection.h"
 #include "models/commonmodel.h"
 #include "models/kservicemodel.h"
@@ -34,29 +33,26 @@
 #include <QGraphicsSceneMouseEvent>
 #include <QGraphicsAnchorLayout>
 #include <QGraphicsView>
-#include <QApplication>
-#include <QFileInfo>
 #include <QListView>
+#include <QVBoxLayout>
 
 #include <KAction>
 #include <KDebug>
 #include <KIcon>
 #include <KIconLoader>
-#include <KLineEdit>
-#include <KStandardDirs>
 #include <KService>
-#include <KServiceTypeTrader>
 #include <KConfigDialog>
-#include <KDesktopFile>
 #include <KRun>
+#include <KLineEdit>
+#include <KKeySequenceWidget>
 
+#include <Plasma/AbstractToolBox>
 #include <Plasma/Theme>
 #include <Plasma/Frame>
 #include <Plasma/Corona>
 #include <Plasma/LineEdit>
 #include <Plasma/IconWidget>
 #include <Plasma/RunnerManager>
-#include <Plasma/QueryMatch>
 #include <Plasma/ScrollWidget>
 #include <Plasma/ToolButton>
 
@@ -104,21 +100,24 @@ void SearchLaunch::init()
 
     connect(this, SIGNAL(toolBoxVisibilityChanged(bool)), this, SLOT(updateConfigurationMode(bool)));
 
-    m_toolBox = new NetToolBox(this);
-    setToolBox(m_toolBox);
-    connect(m_toolBox, SIGNAL(toggled()), this, SIGNAL(toolBoxToggled()));
-    connect(m_toolBox, SIGNAL(visibilityChanged(bool)), this, SIGNAL(toolBoxVisibilityChanged(bool)));
-    m_toolBox->show();
+
+    setToolBox(Plasma::AbstractToolBox::load(corona()->preferredToolBoxPlugin(Plasma::Containment::DesktopContainment), QVariantList(), this));
 
     QAction *a = action("add widgets");
     if (a) {
-        m_toolBox->addTool(a);
+        addToolBoxAction(a);
     }
 
 
+    if (toolBox()) {
+        connect(toolBox(), SIGNAL(toggled()), this, SIGNAL(toolBoxToggled()));
+        connect(toolBox(), SIGNAL(visibilityChanged(bool)), this, SIGNAL(toolBoxVisibilityChanged(bool)));
+        toolBox()->show();
+    }
+
     a = action("configure");
     if (a) {
-        m_toolBox->addTool(a);
+        addToolBoxAction(a);
         a->setText(i18n("Configure Search and Launch"));
     }
 
@@ -136,7 +135,7 @@ void SearchLaunch::init()
         QObject::connect(lockAction, SIGNAL(triggered(bool)), this, SLOT(toggleImmutability()));
     }
 
-    m_toolBox->addTool(lockAction);
+    addToolBoxAction(lockAction);
 
     //FIXME: two different use cases for the desktop and the newspaper, another reason to move the toolbox management out of here
     QAction *activityAction = 0;
@@ -144,7 +143,7 @@ void SearchLaunch::init()
         activityAction = corona()->action("manage activities");
     }
     if (activityAction) {
-        m_toolBox->addTool(activityAction);
+        addToolBoxAction(activityAction);
     }
 
     a = new QAction(i18n("Next activity"), this);
@@ -241,13 +240,15 @@ void SearchLaunch::init()
     setFormFactorFromLocation(location());
 
     if (action("remove")) {
-        m_toolBox->addTool(action("remove"));
+        addToolBoxAction(action("remove"));
     }
 }
 
 void SearchLaunch::launchPackageManager()
 {
-    m_toolBox->setShowing(false);
+    if (toolBox()) {
+        toolBox()->setShowing(false);
+    }
     KRun::run(*m_packageManagerService.data(), KUrl::List(), 0);
 }
 
@@ -269,7 +270,7 @@ void SearchLaunch::configChanged()
             addAction("add applications", addApplicationsAction);
             addApplicationsAction->setText(i18n("Add applications"));
             addApplicationsAction->setIcon(KIcon("applications-other"));
-            m_toolBox->addTool(addApplicationsAction);
+            addToolBoxAction(addApplicationsAction);
 
             connect(addApplicationsAction, SIGNAL(triggered()), this, SLOT(launchPackageManager()));
         }
@@ -350,9 +351,10 @@ void SearchLaunch::launch(QModelIndex index)
     if (m_resultsView->model() == m_runnerModel) {
         KRunnerItemHandler::openUrl(url);
         reset();
+        emit releaseVisualFocus();
     } else {
         QString id = url.path();
-        if (id.startsWith(QLatin1String("/"))) {
+        if (id.startsWith(QLatin1Char('/'))) {
             id = id.remove(0, 1);
         }
         if (url.protocol() == "kservicegroup") {
@@ -363,6 +365,7 @@ void SearchLaunch::launch(QModelIndex index)
         } else {
             KServiceItemHandler::openUrl(url);
             reset();
+            emit releaseVisualFocus();
         }
     }
 
@@ -460,7 +463,7 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
             m_serviceModel = new KServiceModel(config(), this);
             m_resultsView->setModel(m_serviceModel);
         }
-        
+
         resize(corona()->screenGeometry(screen()).size());
     }
 
@@ -516,10 +519,10 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
         }
 
         //kill or create the config overlay if needed
-        if (immutability() == Plasma::Mutable && !m_appletOverlay && m_toolBox->isShowing()) {
+        if (immutability() == Plasma::Mutable && !m_appletOverlay && toolBox() && toolBox()->isShowing()) {
             m_appletOverlay = new LinearAppletOverlay(this, m_appletsLayout);
             m_appletOverlay->resize(size());
-        } else if (immutability() != Plasma::Mutable && m_appletOverlay && m_toolBox->isShowing()) {
+        } else if (immutability() != Plasma::Mutable && m_appletOverlay && toolBox() && toolBox()->isShowing()) {
             m_appletOverlay->deleteLater();
             m_appletOverlay = 0;
         }
@@ -571,27 +574,6 @@ void SearchLaunch::restoreStrip()
 
 void SearchLaunch::updateConfigurationMode(bool config)
 {
-    qreal extraLeft = 0;
-    qreal extraTop = 0;
-    qreal extraRight = 0;
-    qreal extraBottom = 0;
-
-    if (config) {
-        switch (m_toolBox->location()) {
-            case Plasma::LeftEdge:
-            extraLeft= m_toolBox->expandedGeometry().width();
-            break;
-        case Plasma::RightEdge:
-            extraRight = m_toolBox->expandedGeometry().width();
-            break;
-        case Plasma::TopEdge:
-            extraTop = m_toolBox->expandedGeometry().height();
-            break;
-        case Plasma::BottomEdge:
-        default:
-            extraBottom = m_toolBox->expandedGeometry().height();
-        }
-    }
 
     if (config && !m_appletOverlay && immutability() == Plasma::Mutable) {
         if (m_appletsLayout->count() == 2) {
@@ -608,8 +590,6 @@ void SearchLaunch::updateConfigurationMode(bool config)
             m_mainLayout->removeItem(m_appletsLayout);
         }
     }
-
-    m_mainLayout->setContentsMargins(extraLeft, extraTop, extraRight, extraBottom);
 }
 
 void SearchLaunch::overlayRequestedDrop(QGraphicsSceneDragDropEvent *event)
@@ -697,28 +677,17 @@ void SearchLaunch::focusInEvent(QFocusEvent *event)
     if (m_searchField) {
         m_searchField->setFocus();
     }
+    if (screen() < 0) {
+        setScreen(0);
+    }
     Containment::focusInEvent(event);
 }
 
 void SearchLaunch::changeEvent(QEvent *event)
 {
     if (event->type() == QEvent::ContentsRectChange) {
-        qreal left, top, right, bottom;
-        getContentsMargins(&left, &top, &right, &bottom);
 
-        //left preferred over right
-        if (left > top && left > right && left > bottom) {
-            m_toolBox->setLocation(Plasma::RightEdge);
-        } else if (right > top && right >= left && right > bottom) {
-            m_toolBox->setLocation(Plasma::LeftEdge);
-        } else if (bottom > top && bottom > left && bottom > right) {
-            m_toolBox->setLocation(Plasma::TopEdge);
-        //bottom is the default
-        } else {
-            m_toolBox->setLocation(Plasma::BottomEdge);
-        }
-
-        if (m_toolBox->isShowing()) {
+        if (toolBox() && toolBox()->isShowing()) {
             updateConfigurationMode(true);
         }
     }
@@ -738,10 +707,35 @@ void SearchLaunch::createConfigurationInterface(KConfigDialog *parent)
     enabledEntries->setModelColumn(0);
     parent->addPage(enabledEntries, i18nc("Title of the page that lets the user choose what entries will be allowed in the main menu", "Main menu"), "view-list-icons");
 
+    QWidget *page = new QWidget;
+    QVBoxLayout *layout = new QVBoxLayout(page);
+
+    if (!m_shortcutEditor) {
+        m_shortcutEditor = new KKeySequenceWidget(page);
+        connect(parent, SIGNAL(applyClicked()), this, SLOT(configDialogFinished()));
+        connect(parent, SIGNAL(okClicked()), this, SLOT(configDialogFinished()));
+    }
+
+    m_shortcutEditor.data()->setKeySequence(globalShortcut().primary());
+    layout->addWidget(m_shortcutEditor.data());
+    layout->addStretch();
+    parent->addPage(page, i18n("Keyboard Shortcut"), "preferences-desktop-keyboard");
+
     connect(parent, SIGNAL(applyClicked()), m_serviceModel, SLOT(saveConfig()));
     connect(parent, SIGNAL(okClicked()), m_serviceModel, SLOT(saveConfig()));
 }
 
+void SearchLaunch::configDialogFinished()
+{
+    if (m_shortcutEditor) {
+        QKeySequence sequence = m_shortcutEditor.data()->keySequence();
+        if (sequence != globalShortcut().primary()) {
+            setGlobalShortcut(KShortcut(sequence));
+            emit configNeedsSaving();
+        }
+    }
+}
+
 void SearchLaunch::saveFavourites()
 {
     KConfigGroup cg = config();
