@@ -25,15 +25,14 @@
 #include <QApplication>
 #include <QPixmapCache>
 #include <QTimer>
-#include <QVBoxLayout>
 #include <QtDBus/QtDBus>
 
-#include <KAction>
 #include <KCrash>
 #include <KDebug>
 #include <KCmdLineArgs>
 #include <KStandardAction>
 #include <KWindowSystem>
+#include <KAction>
 
 #include <ksmserver_interface.h>
 
@@ -500,7 +499,9 @@ void PlasmaApp::checkShadow()
 
 void PlasmaApp::mainContainmentActivated()
 {
-    m_mainView->setWindowTitle(m_mainView->containment()->activity());
+    if (m_mainView->containment()) {
+        m_mainView->setWindowTitle(m_mainView->containment()->activity());
+    }
 
 
     const WId id = m_mainView->effectiveWinId();
@@ -564,7 +565,11 @@ void PlasmaApp::adjustSize(Kephal::Screen *screen)
 
 void PlasmaApp::reserveStruts()
 {
-    if (!m_controlBar || m_autoHideControlBar || !isDesktop()) {
+    if (!m_controlBar) {
+        return;
+    }
+
+    if (m_autoHideControlBar || !isDesktop()) {
         KWindowSystem::setExtendedStrut(m_controlBar->winId(),
                                     0, 0, 0,
                                     0, 0, 0,
@@ -702,6 +707,7 @@ void PlasmaApp::createView(Plasma::Containment *containment)
     connect(containment, SIGNAL(toolBoxVisibilityChanged(bool)),
             this, SLOT(updateToolBoxVisibility(bool)));
 
+
     KConfigGroup viewIds(KGlobal::config(), "ViewIds");
     int defaultId = 0;
     if (containment->containmentType() == Plasma::Containment::PanelContainment && 
@@ -833,10 +839,13 @@ void PlasmaApp::setAutoHideControlBar(bool autoHide)
         return;
     }
 
-    if (autoHide && !m_unHideTimer) {
-        m_unHideTimer = new QTimer(this);
-        m_unHideTimer->setSingleShot(true);
-        connect(m_unHideTimer, SIGNAL(timeout()), this, SLOT(controlBarVisibilityUpdate()));
+    if (autoHide) {
+        if (!m_unHideTimer) {
+            m_unHideTimer = new QTimer(this);
+            m_unHideTimer->setSingleShot(true);
+            connect(m_unHideTimer, SIGNAL(timeout()), this, SLOT(controlBarVisibilityUpdate()));
+        }
+
         m_controlBar->installEventFilter(this);
         controlBarVisibilityUpdate();
     } else {
@@ -926,16 +935,14 @@ void PlasmaApp::showWidgetExplorer(Plasma::Containment *containment)
         m_widgetExplorerView->installEventFilter(this);
     }
 
-    const bool vert = m_controlBar->location() == Plasma::LeftEdge ||
-                      m_controlBar->location() == Plasma::RightEdge;
-    m_widgetExplorer->setOrientation(vert ? Qt::Vertical : Qt::Horizontal);
+    m_widgetExplorer->setLocation(m_controlBar->location());
 
-    if (m_widgetExplorer->orientation() == Qt::Horizontal) {
-        m_widgetExplorer->setMinimumWidth(m_mainView->size().width());
-        m_widgetExplorer->setMinimumHeight(-1);
-    } else {
+    if (m_widgetExplorer->location() == Plasma::LeftEdge || m_widgetExplorer->location() == Plasma::RightEdge) {
         m_widgetExplorer->setMinimumWidth(-1);
         m_widgetExplorer->setMinimumHeight(m_mainView->size().height());
+    } else {
+        m_widgetExplorer->setMinimumWidth(m_mainView->size().width());
+        m_widgetExplorer->setMinimumHeight(-1);
     }
 
     positionPanel();
@@ -1048,7 +1055,9 @@ bool PlasmaApp::x11EventFilter(XEvent *event)
         QTimer::singleShot(100, this, SLOT(lowerMainView()));
     } else if (m_controlBar && m_autoHideControlBar && m_controlBar->isVisible() &&
         (event->xany.send_event != True && event->type == LeaveNotify)) {
-        m_unHideTimer->start(200);
+        if (m_unHideTimer) {
+            m_unHideTimer->start(200);
+        }
     }
 
     return KUniqueApplication::x11EventFilter(event);
@@ -1153,10 +1162,10 @@ void PlasmaApp::setControlBarVisible(bool visible)
     if (visible) {
         destroyUnHideTrigger();
         Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
-        KWindowSystem::setOnAllDesktops(m_controlBar->effectiveWinId(), m_isDesktop);
         m_controlBar->setWindowFlags(m_mainView->windowFlags() | Qt::FramelessWindowHint);
         m_controlBar->setFrameShape(QFrame::NoFrame);
         m_controlBar->show();
+        KWindowSystem::setOnAllDesktops(m_controlBar->winId(), m_isDesktop);
         unsigned long state = NET::Sticky | NET::StaysOnTop | NET::KeepAbove;
         KWindowSystem::setState(m_controlBar->effectiveWinId(), state);
         KWindowSystem::setType(m_controlBar->effectiveWinId(), NET::Dock);
